from CvPythonExtensions import *
import PyHelpers
import sdToolKit as sd
import math
gc = CyGlobalContext()





class Disasters:
    
    def __init__(self):
        ## Globals
        self.iBaseDisasterChance = 10 ## chance that a disaster will occur at all
        
        
    def loadXML(self, xmlFile):
        if sd.sdEntityExists('Disasters', 'DisasterBonesList'):
            pDisasterBonesList = sd.sdGetVal('Disasters','DisasterBonesList', 'DisasterBoneses')
            for line in xmlFile.readlines():
                if "##" in line: continue
                if "<Disaster>" in line:
                    pDisaster = DisasterBones()
                elif "<DisasterTag>" in line:
                    pDisaster.sDisasterTag = self.CutString(line)
                elif "<iWeight>" in line:
                    pDisaster.iWeight = int(self.CutString(line))
                elif "<PrimaryEffect>" in line:
                    pDisaster.sPrimaryEffect = self.CutString(line)
                elif "<iPrimaryEffectTimer>" in line:
                    pDisaster.iPrimaryEffectTimer = int(self.CutString(line))
                elif "<SecondaryEffect>" in line:
                    pDisaster.sSecondaryEffect = self.CutString(line)
                elif "<iSecondaryEffectTimer>" in line:
                    pDisaster.iSecondaryEffectTimer = int(self.CutString(line))
                elif "</Disaster>" in line:
                    pDisasterBonesList.append(pDisaster)
            sd.sdSetVal('Disasters', 'DisasterBonesList', 'DisasterBoneses', pDisasterBonesList)
            
            
    
    def initializeDisasterList(self):
        sd.sdEntityInit('Disasters', 'DisasterList', {'Disasters': []})
        sd.sdEntityInit('Disasters','DisasterBonesList',{'DisasterBoneses': []})
        MyFile=open("Mods/Disasters/Assets/XML/DisasterInfos.xml")
        
        self.loadXML(MyFile)
        MyFile.close()

    def spawnDisasterByType(self, sDisasterType, x, y):
        
        lDisasterBoneses = sd.sdGetVal('Disasters','DisasterBonesList', 'DisasterBoneses')
        for pDisasterBones in lDisasterBoneses:
            if pDisasterBones.sDisasterTag == sDisasterType:
                self.spawnDisasterByBones(pDisasterBones, x, y)
                

    def spawnDisasterByBones(self, pDisasterBones, x, y):
        pDisaster = Disaster(pDisasterBones)
        pDisaster.X = x
        pDisaster.Y = y
        if sd.sdEntityExists('Disasters', 'DisasterList'):
            lDisasterList = sd.sdGetVal('Disasters', 'DisasterList', 'Disasters')
            lDisasterList.append(pDisaster)
            sd.sdSetVal('Disasters', 'DisasterList', 'Disasters', lDisasterList)
    

    def doDisasterArt(self, iCounter):
        if sd.sdEntityExists('Disasters','DisasterList'):
            lDisasters = sd.sdGetVal('Disasters', 'DisasterList', 'Disasters')
            for pDisaster in lDisasters:
                if (iCounter % pDisaster.iPrimaryEffectTimer == 0 or iCounter == 1) and pDisaster.sPrimaryEffect != "NONE":
                    CyEngine().triggerEffect(gc.getInfoTypeForString(pDisaster.sPrimaryEffect), gc.getMap().plot(pDisaster.X,pDisaster.Y).getPoint())    
                if (iCounter % pDisaster.iSecondaryEffectTimer == 0 or iCounter == 1) and pDisaster.sSecondaryEffect != "NONE":
                    CyEngine().triggerEffect(gc.getInfoTypeForString(pDisaster.sSecondaryEffect), gc.getMap().plot(pDisaster.X, pDisaster.Y).getPoint())
                
                
    def checkSpawning(self):
        pTile = gc.getMap().plot(CyGame().getSorenRandNum(gc.getMap().getGridWidth(), ""),CyGame().getSorenRandNum(gc.getMap().getGridHeight(), ""))
        iDisasterYes = CyGame().getSorenRandNum(100, "chance to disaster")
        if iDisasterYes <= self.iBaseDisasterChance:
            lDisasterBoneses = sd.sdGetVal('Disasters','DisasterBonesList', 'DisasterBoneses')
            lWeights = []
            for pDisasterBones in lDisasterBoneses:
                iWeight = pDisasterBones.iWeight
                lWeights.append(iWeight)
            iTotalWeight = 0
            for i in range(len(lWeights)):
                iTotalWeight += lWeights[i]
            iWhich = CyGame().getSorenRandNum(iTotalWeight, "")
            iSoFarSum = 0
            for i in range(len(lWeights)):
                iSoFarSum += lWeights[i]
                if iWhich <= iSoFarSum:
                    self.spawnDisasterByBones(lDisasterBoneses[i], pTile.getX(), pTile.getY())
                
        
        
    def updateDisasters(self):
        if sd.sdEntityExists('Disasters', 'DisasterList'):
            lDisasters = sd.sdGetVal('Disasters', 'DisasterList', 'Disasters')
            for pDisaster in lDisasters:
                ##self.doDamage(pDisaster)
                ##self.doSpreading(pDisaster)
                ##self.doExtinguishing(pDisaster)
                doNothingYet = True
            sd.sdSetVal('Disasters', 'DisasterList', 'Disasters', lDisasters)
        
    ## This next method was pilfered from The_J's SpawnACiv python mod. 
    ## I used that mod to learn how to load XML and this was a very nifty method to help with that. 
    ## It just cuts out the value of a line in XML and returns it. 
    def CutString(self, string):
        string = str(string)        
        string = string.strip()        
        string = string[2:-1]
        BeginPos=-1
        EndPos = -1                
        for i in xrange(len(string)):
            if string[i]==">":
                BeginPos=i                                
            elif string[i]=="<":
                EndPos=i                                
                break
        else:
            return "-1"
        NewString = string[BeginPos+1:EndPos]        
        return str(NewString)    
    
    
class DisasterBones(object):
    def __init__(self):
        ## This class is loaded as the XML setup for any particular disaster. 
        self.sDisasterTag = ""
        self.sDisasterDescription = ""
        self.iWeight = 0
        self.sPrimaryEffect = ""
        self.iPrimaryEffectTimer = -1
        self.sSecondaryEffect = ""
        self.iSecondaryEffectTimer = -1
    
class Disaster(object):
    def __init__(self, pDisasterBones):
        self.sDisasterTag = pDisasterBones.sDisasterTag
        self.sDisasterDescription = pDisasterBones.sDisasterDescription
        self.X = -1
        self.Y = -1
        self.sPrimaryEffect = pDisasterBones.sPrimaryEffect
        self.iPrimaryEffectTimer = pDisasterBones.iPrimaryEffectTimer
        self.sSecondaryEffect = pDisasterBones.sSecondaryEffect
        self.iSecondaryEffectTimer = pDisasterBones.iSecondaryEffectTimer